﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [StorableClass]
  [Item("GenealogyGraphArc", "A graph arc connecting two GenealogyGraphNodes and holding some data.")]
  public class GenealogyGraphArc : Arc, IGenealogyGraphArc {
    [StorableConstructor]
    protected GenealogyGraphArc(bool deserializing) : base(deserializing) { }
    protected GenealogyGraphArc(GenealogyGraphArc original, Cloner cloner)
      : base(original, cloner) { }

    public GenealogyGraphArc(IGenealogyGraphNode source, IGenealogyGraphNode target)
      : base(source, target) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GenealogyGraphArc(this, cloner);
    }

    public new IGenealogyGraphNode Source {
      get { return (IGenealogyGraphNode)base.Source; }
    }

    public new IGenealogyGraphNode Target {
      get { return (IGenealogyGraphNode)base.Target; }
    }
  }

  [StorableClass]
  [Item("GenealogyGraphArc", "")]
  public class GenealogyGraphArc<T> : GenealogyGraphArc where T : class,IItem {
    [StorableConstructor]
    protected GenealogyGraphArc(bool deserializing) : base(deserializing) { }
    protected GenealogyGraphArc(GenealogyGraphArc original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GenealogyGraphArc<T>(this, cloner);
    }
    public new T Data {
      get { return (T)base.Data; }
      set { base.Data = value; }
    }
  }
}
